
/*
 * process.cc
 * Copyright (C) 1999 by John Heidemann
 * $Id: process_view.cc,v 1.20 1999/05/21 16:55:08 johnh Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <stdio.h>  // snprintf

#include "main.hh"
#include "process_view.hh"
#include "process_model.hh"



process_view::process_view(process_model *pm) : pm_(pm)
{
	ENTRY_TRACE(__FILE__,__LINE__);
	size_.tick(scale_size(pm_->size_.last()));
	time_.tick(scale_time(pm_->utime_.last() + pm_->stime_.last()));
	b_ = blob::create_real_blob(this);
	reset_color();
	init_position();
	b_->set_size(size_.last());
}

process_view::~process_view()
{
	ENTRY_TRACE(__FILE__,__LINE__);
	delete b_;
}

void
process_view::reset_color()
{
	ENTRY_TRACE(__FILE__,__LINE__);
	S_ = 0.6;
	B_ = 0.8 - (pm_->age() * 0.06);
	const double MIN_B = 0.15;
	if (B_ < MIN_B)
		B_ = MIN_B;

	H_ = (pm_->cmd_hash_.last() % 1024) / 1024.0;

	b_->set_hsb(H_, S_, B_);
}

void
process_view::init_position()
{
	ENTRY_TRACE(__FILE__,__LINE__);
	int x = pm_->pid_ % (blob::X_MAX - blob::INITIAL_SLOP);
	int y = pm_->uid_ % (blob::Y_MAX - blob::INITIAL_SLOP);
	b_->set_position(x, y);
}

int
process_view::scale_size(int size)
{
	ENTRY_TRACE(__FILE__,__LINE__);
	return size / 100;  // xxx: should by dynamic
}

int
process_view::scale_time(int time)
{
	ENTRY_TRACE(__FILE__,__LINE__);
	// covert from usec to 100ths of a second
	return time / 10000;
}

void
process_view::update()
{
	ENTRY_TRACE(__FILE__,__LINE__);
	age_.tick(pm_->age());
	if (age_.changed() || pm_->cmd_hash_.changed())
		reset_color();

	if (pm_->size_.changed()) {
		size_.tick(scale_size(pm_->size_.last()));
		if (size_.changed())
			b_->grow(size_.change());
	};

	time_.tick(scale_time(pm_->utime_.last() + pm_->stime_.last()));
	if (time_.changed())
		b_->move(time_.change());

}


void
process_view::info_to_buf(char *buf, int len)
{
	ENTRY_TRACE(__FILE__,__LINE__);
	// xxx: should be snprintf
	snprintf(buf, len, "user: %d\npid: %d\ncmd: %s\nnice: %d\nmem: %dKB\n",
		 pm_->uid_,
		 pm_->pid_, pm_->cmd_.c_str(), pm_->nice_, pm_->size_.last());
}

process_view *
process_view::superior()
{
	process_model *superior_model =  pm_->superior();
	return superior_model ? superior_model->view() : NULL;
}
